#! /usr/bin/perl
#
# (C)2006 Luca Corti, Lorenzo Vigano' - (C) 2007 Luca Corti
# $Id: check_kannel.pl,v 1.1.1 2007/09/03 15:36:23 lcorti Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

require 5.004;
use strict;
use POSIX;
use LWP::UserAgent;
use XML::DOM;
use XML::XPath;
use XML::XPath::XMLParser;
use Getopt::Long;
use vars qw($opt_V $opt_v $opt_P $opt_p $opt_h $opt_H $opt_s $opt_t $opt_w $opt_c $opt_i $opt_j $opt_k $opt_o $opt_q $opt_r $opt_S $opt_W %kannel %plugin $response $ua $xp $PROGNAME $PROGREV);

my $TIMEOUT = 20;


$PROGNAME="check_kannel_cacti.pl";
$PROGREV='$Revision: 1.1.1 $';

$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

sub print_output($);
sub print_help();
sub print_usage();

# Plugin data
%plugin = (
	hostname    => 'localhost',
	password    => '',
	port        => '13000',
	timeout     => "$TIMEOUT",
	url         => '',
	verbose     => '0',
	check       => {
		smsbox => 0,
		wapbox => 0
	},
	thresholds  => {
		sms => {
			received => {
				total => {
					warning  => 0,
					critical => 0
				},
				queued => {
					warning  => 0,
					critical => 0
				}
			},
			sent => {
				total => {
					warning  => 0,
					critical => 0
				},
				queued => {
					warning  => 0,
					critical => 0
				}
			}
		},
		dlr => {
			queued => {
				warning  => 0,
				critical => 0
			}
		},
		boxes => {
			smsbox => {
				online => {
					warning  => 1,
					critical => 0
				}
			},
			wapbox => {
				online => {
					warning  => 1,
					critical => 0
				}
			}
		},
		smscs => {
			online => {
				warning  => 1,
				critical => 0
			}
		}
	}
);

# Kannel status.xml imported data structure
%kannel = (
	version => '',
	status  => '',
	wdp => {
		received => {
			total  => 0,
			queued => 0
		},
		sent => {
			total  => 0,
			queued => 0
		},
	},
	sms => {
		received => {
			total  => 0,
			queued => 0
		},
		sent => {
			total  => 0,
			queued => 0
		},
		storesize => '',
		inbound   => '',
		outbound  => ''
	},
	dlr  => {
		queued  => '',
		storage => ''
	},
	boxes => {
		smsbox => {
			count  => 0,
			online => 0
		},
		wapbox => {
			count  => 0,
			online => 0
		}
	},
	smscs => {
		count  => '',
		online => 0
	}
);

Getopt::Long::Configure('bundling');
GetOptions
	("V"   => \$opt_V, "version"        => \$opt_V,
	 "h"   => \$opt_h, "help"           => \$opt_h,
	 "v"   => \$opt_v, "verbose"        => \$opt_v,
	 "s"   => \$opt_s, "secure"         => \$opt_s,
	 "S"   => \$opt_S, "smsbox"         => \$opt_S,
	 "W"   => \$opt_W, "wapbox"         => \$opt_W,
	 "t=i" => \$opt_t, "timeout=i"      => \$opt_t,
	 "H=s" => \$opt_H, "host=s"         => \$opt_H,
	 "P=i" => \$opt_P, "port=i"         => \$opt_P,
	 "p=s" => \$opt_p, "password=s"     => \$opt_p);

if ($opt_V)
{
	print_revision($PROGNAME,$PROGREV);
	exit;
}

if ($opt_h)
{
	print_help();
	exit;
}

if ($opt_v)
{
	$plugin{verbose} = 1;
}

if ($opt_H and $opt_H =~ m/^([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[a-zA-Z][-a-zA-Z0-9]+(\.[a-zA-Z][-a-zA-Z0-9]+)*)$/)
{
	$plugin{hostname} = $opt_H;
}

if ($opt_P and $opt_P =~ /^([0-9]+)$/)
{
	$plugin{port} = $1;
}

if ($opt_p)
{
	$plugin{password} = $opt_p;
}

if ($opt_t and $opt_t =~ /^([0-9]+)$/)
{
	$plugin{timeout} = $1;
}

if ($opt_S)
{
	$plugin{check}{smsbox} = 1;
}

if ($opt_W)
{
	$plugin{check}{wapbox} = 1;
}

if ($opt_w and $opt_w =~ /^([0-9]+)$/)
{
	$plugin{thresholds}{smscs}{online}{warning} = $1;
}

if ($opt_c and $opt_c =~ /^([0-9]+)$/)
{
	$plugin{thresholds}{smscs}{online}{critical} = $1;
}

if ($opt_i and $opt_i =~ /^([0-9]+)$/)
{
	$plugin{thresholds}{sms}{received}{queued}{warning} = $1;
}

if ($opt_j and $opt_j =~ /^([0-9]+)$/)
{
	$plugin{thresholds}{sms}{received}{queued}{warning} = $1;
}

if ($opt_o and $opt_o =~ /^([0-9]+)$/)
{
	$plugin{thresholds}{sms}{sent}{queued}{warning} = $1;
}

if ($opt_q and $opt_q =~ /^([0-9]+)$/)
{
	$plugin{thresholds}{sms}{sent}{queued}{critical} = $1;
}

# Just in case of problems, let's not hang Nagios
$SIG{'ALRM'} = sub
{
	#print ("UNKNOWN: No response from Kannel server (timeout)\n");
	exit;
};

alarm($plugin{timeout});

$plugin{url}="http://".$plugin{hostname}.":".$plugin{port}."/status.xml?password=".$plugin{password};
$plugin{verbose} and print"Will try to get status from ".$plugin{url}."...";

# Get status.xml from server
$ua = LWP::UserAgent->new;
if ($opt_s)
{
	$ua->protocols_allowed([ 'https' ]);
}
else
{
	$ua->protocols_allowed([ 'http' ]);
}
$ua->timeout(10);
$ua->env_proxy;
 
$response = $ua->get($plugin{url});

if ($response->is_success)
{
	$plugin{verbose} and print " got response from server!\n" and print "\n".$response->content."\n";
}
else
{
	#print "CRITICAL: Can't connect to Kannel: ".$response->status_line."\n";
	exit;
}
 
$xp = XML::XPath->new(xml => $response->content);

if ($xp->getNodeText("/gateway") =~ /Denied/)
{
	#print "UNKNOWN: Can't connect to Kannel: Access Denied (wrong password?)\n";
	exit;
}

$kannel{status} = $xp->getNodeText("/gateway/status");
($kannel{status}, $kannel{uptime}) = split(/,/,$kannel{status});
$kannel{sms}{received}{queued} = $xp->getNodeText("/gateway/sms/received/queued");
$kannel{sms}{sent}{queued} = $xp->getNodeText("/gateway/sms/sent/queued");
$kannel{sms}{inbound} = $xp->getNodeText("/gateway/sms/inbound");
$kannel{sms}{outbound} = $xp->getNodeText("/gateway/sms/outbound");
$kannel{sms}{storesize} = $xp->getNodeText("/gateway/sms/storesize");
$kannel{dlr}{queued} = $xp->getNodeText("/gateway/dlr/queued");
$kannel{dlr}{storage} = $xp->getNodeText("/gateway/dlr/storage");
$kannel{smscs}{count} = $xp->getNodeText("/gateway/smscs/count");

# Get number of total/online sms/wap boxes
foreach my $box ($xp->findnodes("/gateway/boxes/box"))
{

	my $type = $box->findnodes("type");
	my $status = $box->findnodes("status");

	# smsboxes
	if ($type->string_value eq "smsbox")
	{
		$kannel{boxes}{smsbox}{count}++;
		
		if ($status->string_value =~ /^on-line/)
		{
			$kannel{boxes}{smsbox}{online}++;
		}
	# wapboxes
	}
	elsif ($type->string_value eq "wapbox")
	{
		$kannel{boxes}{wapbox}{count}++;
		if ($status->string_value =~ /^on-line/)
		{
			$kannel{boxes}{wapbox}{online}++;
		}
	}

}

# Get number of total/online smscs
foreach my $smsc ($xp->findnodes("/gateway/smscs/smsc/status"))
{
	if ($smsc->string_value =~ /^online/)
	{
		$kannel{smscs}{online}++;
	}
}

# Kannel running
if ($kannel{status} eq 'running')
{
	print "sms_recieved_queued:".$kannel{sms}{received}{queued}." ";
	print "sms_sent_queued:".$kannel{sms}{sent}{queued}." ";
	print "sms_inbound:".$kannel{sms}{inbound}." ";
	print "sms_outbound:".$kannel{sms}{outbound}." ";
	print "sms_storesize:".$kannel{sms}{storesize}." ";
	print "dir_queued:".$kannel{dlr}{queued}." ";
	print "dir_storage:".$kannel{dlr}{storage}." ";
	print "smscs_count:".$kannel{smscs}{count}." ";
	print "boxes_wapbox_online:".$kannel{boxes}{wapbox}{online}." ";
	print "boxes_smsbox_online:".$kannel{boxes}{smsbox}{online};
	exit;
}
else
{
	exit;
}

#### subs

sub print_output($)
{
	my $state = shift;
	print "Kannel $state - $kannel{status},$kannel{uptime}, $kannel{boxes}{smsbox}{online}/$kannel{boxes}{smsbox}{count} SMSBOX, $kannel{boxes}{wapbox}{online}/$kannel{boxes}{wapbox}{count} WAPBOX, $kannel{smscs}{online}/$kannel{smscs}{count} SMSC, $kannel{sms}{received}{queued}/$kannel{sms}{sent}{queued} SMS queued in/out, $kannel{dlr}{queued} DLR queued ($kannel{dlr}{storage}), store-file $kannel{sms}{storesize} bytes\n";
}

sub print_usage()
{
	print "Usage: $PROGNAME [ OPTIONS ]\n";
}

sub print_help()
{
	print "Copyright (c) 2006 Luca Corti, Lorenzo Vigano - 2007 Luca Corti\n";
	print "$PROGNAME, $PROGREV\n";
	print "Checks the status of the process, online sms and wap boxes, connected SMSCs, SMS queues and delivery rate for a kannel instance via the status.xml interface.";
	print "\n";
	print_usage();
	print "Checks the Kannel, configured SMSCs and SMS received/sent queues/rates on <host> via the status.xml interface.
-t (--timeout) 
     Timeout in seconds at which an unknown status will be generated.
     Defaults to $TIMEOUT.
-s (--secure)
     Connect to the serverly using HTTPS instead of HTTP.
     Defaults to false. (HTTP)
-H (--host)
     Hostname of the Kannel server on which the status interface is running.
     Defaults to localhost.
-P (--port)
     Port number at which the status interface is reachable on the server.
     Defaults to 13000.
-p (--password)
     Password needed to access the status interface.
     Defaults to blank (no password).
\n";	

}
